<?php

namespace Drupal\transform_api\Plugin\Transform\Block;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\transform_api\FieldTransformManager;
use Drupal\transform_api\Transform\EntityTransform;
use Drupal\transform_api\Transform\RouteTransform;
use Drupal\transform_api\TransformBlockBase;
use Drupal\transform_api\TransformBlocks;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\RouterInterface;

/**
 * Provides a 'Main page content' block.
 *
 * @TransformBlock(
 *   id = "system_main_block",
 *   admin_label = @Translation("Main page content"),
 *   category = @Translation("System"),
 * )
 */
class SystemMainTransformBlock extends TransformBlockBase {

  /**
   * @var ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * @var \Symfony\Component\Routing\RouterInterface
   */
  protected RouterInterface $router;

  /**
   * @var \Drupal\transform_api\FieldTransformManager
   */
  protected FieldTransformManager $fieldTransformManager;

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * @var \Drupal\transform_api\TransformBlocks
   */
  private TransformBlocks $transformBlocks;

  function __construct(array $configuration, $plugin_id, $plugin_definition, RouterInterface $router, ConfigFactoryInterface $configFactory, TransformBlocks $transformBlocks, RequestStack $requestStack) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->router = $router;
    $this->configFactory = $configFactory;
    $this->requestStack = $requestStack;
    $this->transformBlocks = $transformBlocks;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('router'),
      $container->get('config.factory'),
      $container->get('transform_api.transform_blocks'),
      $container->get('request_stack')
    );
  }

  public function transform() {
    $url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost() . $this->requestStack->getCurrentRequest()->getBaseUrl() . $this->requestStack->getCurrentRequest()->getPathInfo();
    $match = $this->router->match($url);
    $route_entity = $this->getEntityFromMatchArray($match);
    $cacheMetadata = new CacheableMetadata();
    $cacheMetadata->setCacheContexts(['route']);

    if (!is_null($route_entity) &&
      $route_entity instanceof ContentEntityInterface &&
      (substr($match['_route'], 0, 7) == 'entity.') &&
      (substr($match['_route'], -10) == '.canonical')) {
      $cacheMetadata->addCacheableDependency($route_entity);
      $transform = [
        '#collapse' => TRUE,
        'content' => new EntityTransform($route_entity->getEntityTypeId(), $route_entity->id(), 'full')
      ];
    } else {
      /** @var ParameterBag $parameter_bag */
      $parameter_bag = $match['_raw_variables'];
      $transform = [
        '#collapse' => TRUE,
        'content' => new RouteTransform($match['_route'], $parameter_bag->all())
      ];
    }
    $cacheMetadata->applyTo($transform);
    return $transform;
  }

  /**
   * Returns an entity parameter from a route match object.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return mixed|null
   *   The entity, or null if it's not an entity route.
   */
  protected function getEntityFromMatchArray(array $array) {
    $route = $array['_route_object'] ?? NULL;
    if (!$route) {
      return NULL;
    }

    $entity_type_id = $this->getEntityTypeFromRoute($route);
    if ($entity_type_id) {
      return $array[$entity_type_id];
    }

    return NULL;
  }

  /**
   * Return the entity type id from a route object.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route object.
   *
   * @return string|null
   *   The entity type id, null if it doesn't exist.
   */
  protected function getEntityTypeFromRoute(\Symfony\Component\Routing\Route $route): ?string {
    if (!empty($route->getOptions()['parameters'])) {
      foreach ($route->getOptions()['parameters'] as $option) {
        if (isset($option['type']) && strpos($option['type'], 'entity:') === 0) {
          return substr($option['type'], strlen('entity:'));
        }
      }
    }

    return NULL;
  }

}
