<?php

namespace Drupal\transform_api\Transform;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityInterface;
use Drupal\transform_api\Configs\EntityTransformDisplayInterface;
use Drupal\transform_api\FieldTransformInterface;
use Drupal\transform_api\Repository\EntityTransformRepositoryInterface;

class EntityTransform extends PluginTransformBase {

  /**
   * @var \Drupal\Core\Entity\EntityInterface|null
   */
  protected EntityInterface|null $entity = NULL;

  /**
   * @var \Drupal\transform_api\Configs\EntityTransformDisplayInterface|null
   */
  protected EntityTransformDisplayInterface|null $display = NULL;

  /**
   * @var FieldTransformInterface[]
   */
  protected array $fields = [];

  /**
   * @param string $entity_type
   * @param int|string|array $ids
   * @param string $transform_mode
   */
  public function __construct($entity_type, $ids, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE) {
    $this->values = [
      'entity_type' => $entity_type,
      'ids' => $ids,
      'transform_mode' => $transform_mode
    ];
    if (is_array($ids)) {
      foreach ($ids as $id) {
        $this->cacheTags[] = $entity_type . ':' . $id;
      }
    } else {
      $this->cacheTags[] = $entity_type . ':' . $ids;
    }
  }

  /**
   * @inheritDoc
   */
  public function getTransformType() {
    return 'entity';
  }

  public function getAlterIdentifiers() {
    return [$this->getTransformType(), $this->values['entity_type']];
  }

  /**
   * @inheritDoc
   */
  public function isMultiple() {
    return is_array($this->getValue('ids'));
  }

  /**
   * @inheritDoc
   */
  public function shouldBeCached() {
    return !$this->isMultiple();
  }

  /**
   * @return \Drupal\Core\Entity\EntityInterface|null
   */
  public function getEntity(): ?EntityInterface {
    return $this->entity;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface|null $entity
   */
  public function setEntity(?EntityInterface $entity): void {
    $this->entity = $entity;
  }

  /**
   * @return FieldTransformInterface[]
   */
  public function getFields(): array {
    return $this->fields;
  }

  /**
   * @param FieldTransformInterface[] $fields
   */
  public function setFields(array $fields): void {
    $this->fields = $fields;
  }

  /**
   * @return \Drupal\transform_api\Configs\EntityTransformDisplayInterface|null
   */
  public function getDisplay(): ?EntityTransformDisplayInterface {
    return $this->display;
  }

  /**
   * @param \Drupal\transform_api\Configs\EntityTransformDisplayInterface|null $display
   */
  public function setDisplay(?EntityTransformDisplayInterface $display): void {
    $this->display = $display;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param FieldTransformInterface[] $fields
   * @param EntityTransformDisplayInterface $display
   *
   * @return EntityTransform
   */
  public static function createPrepared(EntityInterface $entity, array $fields, EntityTransformDisplayInterface $display): EntityTransform {
    $transform = new self($entity->getEntityTypeId(), $entity->id(), $display->getMode());
    $transform->setEntity($entity);
    $transform->setFields($fields);
    $transform->setDisplay($display);
    return $transform;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param string $transform_mode
   *
   * @return EntityTransform
   */
  public static function createFromEntity(EntityInterface $entity, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE) {
    return new self($entity->getEntityTypeId(), $entity->id(), $transform_mode);
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface[] $entities
   * @param string $transform_mode
   *
   * @return EntityTransform
   */
  public static function createFromMultipleEntities(array $entities, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE) {
    $ids = [];
    $entityTypeId = '';
    foreach ($entities as $entity) {
      $ids[] = $entity->id();
      $entityTypeId = $entity->getEntityTypeId();
    }
    return new self($entityTypeId, $ids, $transform_mode);
  }

}
